@@ -6,14 +6,15 @@ from pathlib import Path
 from typing import Sequence, Union
 
 from tarn.digest import digest_file
-from wcmatch import glob
+from wcmatch.glob import GLOBSTAR
 
 from .config import CONFIG, build_storage, find_vcs_root
-from .hash import is_hash, to_hash, load_tree, load_tree_key, strip_tree, FileHash, InsideTreeHash, Key
-from .vc import Version, CommittedVersion, VC, SubprocessGit
+from .exceptions import RepositoryNotFound, HashNotFound, InconsistentHash, NameConflict, InconsistentRepositories
+from .hash import is_hash, to_hash, load_tree, strip_tree, Key, load_key, is_tree
 from .local import Local
-from .exceptions import RepositoryNotFound, HashNotFound, InconsistentHash
-from .utils import InconsistentRepositories, PathLike
+from .utils import PathOrStr
+from .vc import Version, CommittedVersion, VC, SubprocessGit
+from .wc import BevLocalGlob, BevVCGlob
 
 
 class Repository:
@@ -32,19 +33,20 @@ class Repository:
         means that the local (possibly uncommitted) version of the files will be used.
         Can be overridden in corresponding methods
     check: bool
-        default value for `resolve` move. If True - the file's hash will be additionally checked for consistency.
+        default value for `resolve` mode. If True - the file's hash will be additionally checked for consistency.
         Can be overridden in corresponding methods
     """
 
-    def __init__(self, *root: PathLike, fetch: bool = True, version: Version = None, check: bool = False):
+    def __init__(self, *root: PathOrStr, fetch: bool = True, version: Version = None, check: bool = False):
         self.root = Path(*root)
         self.prefix = Path()
         self.storage, self.cache = build_storage(self.root)
         self.vc: VC = SubprocessGit(self.root)
         self._fetch, self._version, self._check = fetch, version, check
+        self._cache = {}
 
     @classmethod
-    def from_here(cls, *relative: PathLike, fetch: bool = True, version: Version = None,
+    def from_here(cls, *relative: PathOrStr, fetch: bool = True, version: Version = None,
                   check: bool = None) -> 'Repository':
         """
         Creates a repository with a path `relative` to the file in which this method is called.
@@ -57,7 +59,7 @@ class Repository:
         return cls(file.parent / Path(*relative), fetch=fetch, version=version, check=check)
 
     @classmethod
-    def from_vcs(cls, *parts: PathLike) -> 'Repository':
+    def from_vcs(cls, *parts: PathOrStr) -> 'Repository':
         vcs = find_vcs_root(Path(os.getcwd(), *parts))
         if vcs is None:
             raise RepositoryNotFound(f'{Path(*parts)} is not inside a vcs repository')
@@ -75,7 +77,7 @@ class Repository:
         warnings.warn('This property is deprecated, use `latest_version()` instead', DeprecationWarning)
         return self.latest_version()
 
-    def latest_version(self, path: PathLike = '.', *, default=inspect.Parameter.empty) -> CommittedVersion:
+    def latest_version(self, path: PathOrStr = '.', *, default=inspect.Parameter.empty) -> CommittedVersion:
         """
         Get the last commit given the relative `path`.
         """
@@ -83,7 +85,7 @@ class Repository:
         if not (self.root / path).exists() and not is_hash(path):
             path = to_hash(path)
 
-        version = self.vc.log(str(path))
+        version = self.vc.get_version(str(path))
         if version is None:
             if default is inspect.Parameter.empty:
                 raise FileNotFoundError(f'The path "{path}" is not present in any commit')
@@ -92,7 +94,7 @@ class Repository:
 
         return version
 
-    def resolve(self, *parts: PathLike, version: Version = None, fetch: bool = None, check: bool = None) -> Path:
+    def resolve(self, *parts: PathOrStr, version: Version = None, fetch: bool = None, check: bool = None) -> Path:
         """
         Get the real path of a file in the repository
 
@@ -106,7 +108,7 @@ class Repository:
             the data version. Can be either a string with a commit hash/tag or the `Local` object, which
             means that the local (possibly uncommitted) version of the files will be used
         check: bool
-            default value for `resolve` move. If True - the file's hash will be additionally checked for consistency
+            default value for `resolve` mode. If True - the file's hash will be additionally checked for consistency
         """
 
         def _resolve(path):
@@ -121,11 +123,17 @@ class Repository:
 
         relative = self._normalize_relative(*parts)
         version = self._resolve_version(version)
+        absolute = self.root / relative
+        if version == Local and absolute.exists():
+            if to_hash(absolute).exists():
+                raise NameConflict(f'Both the path "{relative}" and its hash "{to_hash(relative)}" found')
+            return absolute.resolve()
+
         check = self._resolve_check(check)
         key = self.get_key(relative, version=version, fetch=fetch)
         return self.storage.read(_resolve, key, fetch=self._resolve_fetch(fetch))
 
-    def glob(self, *parts: PathLike, version: Version = None, fetch: bool = None) -> Sequence[Path]:
+    def glob(self, *parts: PathOrStr, version: Version = None, fetch: bool = None) -> Sequence[Path]:
         """
         Get all the paths in the repository that match a given pattern
 
@@ -139,20 +147,20 @@ class Repository:
             the data version. Can be either a string with a commit hash/tag or the `Local` object, which
             means that the local (possibly uncommitted) version of the files will be used
         """
-        relative = self._normalize_relative(*parts)
         version = self._resolve_version(version)
-        h = self._split(relative, version)
-        if not isinstance(h, InsideTreeHash):
-            raise ValueError('`glob` is only applicable to tree hashes')
+        pattern = os.path.join(*parts)
 
-        pattern = str(h.relative)
-        tree = self._load(load_tree, h.key, fetch)
-        files = sorted(self._expand_folders(tree))
+        if version == Local:
+            glob = BevLocalGlob(pattern, self.root, self.prefix, self.storage, fetch, GLOBSTAR)
+        else:
+            glob = BevVCGlob(
+                pattern, self.root, self.prefix, version, self._cache, self.vc, self.storage, fetch, GLOBSTAR
+            )
 
-        return [h.root / file for file in glob.globfilter(files, pattern, flags=glob.GLOBSTAR)]
+        return list(map(Path, glob.glob()))
 
     # TODO: cache this based on path parents
-    def get_key(self, *parts: PathLike, version: Version = None, fetch: bool = None,
+    def get_key(self, *parts: PathOrStr, version: Version = None, fetch: bool = None,
                 error: bool = True) -> Union[Key, None]:
         version = self._resolve_version(version)
         path = self._normalize_relative(*parts)
@@ -163,12 +171,14 @@ class Repository:
                 raise
             return None
 
-        if isinstance(h, FileHash):
-            return h.key
+        if isinstance(h, Key):
+            return h
 
-        assert isinstance(h, InsideTreeHash), h
-        relative = str(h.relative)
-        tree = self._get_tree(h.key, version, fetch)
+        h, relative = h
+        if relative == '.':
+            raise HashNotFound(f'"{str(path)}" is a hashed folder')
+
+        tree = self._get_tree(h, version, fetch)
         if relative not in tree:
             if relative in self._expand_folders(tree):
                 raise HashNotFound(f'"{str(path)}" is a folder inside a tree hash')
@@ -179,7 +189,7 @@ class Repository:
 
         return tree[relative]
 
-    def load_tree(self, path: PathLike, version: Version = None, fetch: bool = None) -> dict:
+    def load_tree(self, path: PathOrStr, version: Version = None, fetch: bool = None) -> dict:
         path = self._normalize_relative(path)
         version = self._resolve_version(version)
         key = self._get_hash(Path(path), version)
@@ -191,7 +201,7 @@ class Repository:
 
     # navigation
 
-    def __truediv__(self, other: Union[str, Path]):
+    def __truediv__(self, other: PathOrStr):
         other = Path(other)
         if other.is_absolute():
             raise ValueError('Only relative paths are supported')
@@ -199,6 +209,7 @@ class Repository:
         child = Repository(self.root, fetch=self._fetch, version=self._version, check=self._check)
         # FIXME
         child.prefix = self.prefix / other
+        child._cache = self._cache
         return child
 
     @property
@@ -208,6 +219,7 @@ class Repository:
     # internal logic
 
     def _normalize_relative(self, *parts):
+        # TODO: check that it's not a hash path
         return self.prefix / Path(*parts)
 
     def _get_tree(self, key, version, fetch):
@@ -228,12 +240,12 @@ class Repository:
     def _get_uncomitted_hash(self, relative: Path):
         path = self.root / relative
         if path.exists():
-            return load_tree_key(path)
+            return load_key(path)
 
     @lru_cache(None)
     def _get_committed_hash(self, relative: Path, version: CommittedVersion):
         assert isinstance(version, CommittedVersion), type(version)
-        return self.vc.show(str(relative), version)
+        return self.vc.read(str(relative), version)
 
     def _load(self, func, key, fetch):
         fetch = self._resolve_fetch(fetch)
@@ -246,14 +258,17 @@ class Repository:
             key = self._get_hash(hash_path, version)
             if key is not None:
                 key = strip_tree(key)
-                return InsideTreeHash(key, parent, hash_path, path.relative_to(parent))
+                return key, str(path.relative_to(parent))
 
         hash_path = to_hash(path)
         key = self._get_hash(hash_path, version)
         if key is None:
             raise HashNotFound(path)
 
-        return FileHash(key, path, hash_path)
+        if is_tree(key):
+            return key, '.'
+
+        return key
 
     def _resolve_check(self, check):
         if check is None:
